/*
 Copyright (C) 2005 Michael Hanselmann
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 $Id: throttle.c,v 1.2 2005/04/30 23:09:20 michael Exp $
*/

#define _XOPEN_SOURCE 500
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include "pmt.h"

static struct timeval tv, last_tv;
static float dataspeed = -1;

PMT_MODULE_INIT(throttle) {
    gettimeofday(&last_tv, NULL);

    for(int i = 0; i < argc; ++i) {
        if(!strncmp(argv[i], "--to=", 5) && strlen(argv[i]) > 5) {
            dataspeed = atof(argv[i] + (5 * sizeof(char)));
            // Calculate B/s
            dataspeed *= 1024;
        }
    }

    if(dataspeed == -1) {
        write(STDERR_FILENO, "Please specify --to=<maxspeed>, see --help.\n", 44);
        return 0;
    }else{
        return 1;
    }
}

PMT_MODULE_IMPL(throttle) {
    if(dataspeed == -1) return;

    gettimeofday(&tv, NULL);

    float duration = TIMEDIFF(last_tv, tv);
    float durmax = len / dataspeed * 1000;

    if(durmax > duration) {
        usleep((durmax - duration) * 1000);
    }

    gettimeofday(&last_tv, NULL);
}

PMT_MODULE_END(throttle) { }
